/**
  ******************************************************************************
  * @file    types.h
  * @author  Milandr Application Team
  * @version V1.0.0
  * @date    26/10/2020
  * @brief   Types and returncodes
  ******************************************************************************
  * <br><br>
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, MILANDR SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2020 Milandr</center></h2>
  */

#ifndef __TYPES_H__
#define __TYPES_H__

#include <stdint.h>
#include <memory>

// wrappers
#define mldrMemSet(dst, byte, sz) memset(dst, byte, sz)
#define mldrMemCpy(dst, src, sz) memcpy(dst, src, sz)
#define mldrMemCmp(lhs, rhs, sz) memcmp(lhs, rhs, sz)

#ifdef ASSERT_MODE
/* Note we need the 2 concats below because arguments to ##
* are not expanded, so we need to expand __LINE__ with one indirection
* before doing the actual concatenation. */
#define ASSERT_CONCAT_(a, b) a##b
#define ASSERT_CONCAT(a, b) ASSERT_CONCAT_(a, b)
#define compiletime_assert(e) enum { ASSERT_CONCAT(assert_line_, __LINE__) = 1/(!!(e)) }
#else
#define compiletime_assert(e)
#endif

typedef uint32_t Sz128;
typedef uint32_t Sz32;
typedef uint32_t Sz8;
typedef uint32_t CM0_STATUS;

// Base CM0 UART/GATE statuses
#define CM0_OK                  ((CM0_STATUS)0x3C5A0000)
#define CM0_FAIL                ((CM0_STATUS)0xC3A50000)

// preinit-init-work statuses
#define CM0_PREINIT_READY       ((CM0_STATUS)(0x5C0F | CM0_OK))
#define CM0_PREINIT_DONE        ((CM0_STATUS)(0xC535 | CM0_OK))
#define CM0_PREINIT_FAIL        ((CM0_STATUS)(0x533A | CM0_FAIL))
#define CM0_INIT_READY          ((CM0_STATUS)(0xF05F | CM0_OK))
#define CM0_INIT_DONE           ((CM0_STATUS)(0xC335 | CM0_OK))
#define CM0_INIT_FAIL           ((CM0_STATUS)(0xFC3A | CM0_FAIL))
#define CM0_GAMMA_READY         ((CM0_STATUS)(0x0A59 | CM0_OK))
#define CM0_WORK_READY          ((CM0_STATUS)(0xF535 | CM0_OK))
#define CM0_WORK_DONE           ((CM0_STATUS)(0xA50A | CM0_OK))

// errorcodes
#define CM0_SN_CRC_ERR          ((CM0_STATUS)(0xCC3F | CM0_FAIL))
#define CM0_UNDEFINED_STATE     ((CM0_STATUS)(0x5C3C | CM0_FAIL))
#define CM0_GAMMA_NOT_READY     ((CM0_STATUS)(0x5ACF | CM0_FAIL))
#define CM0_GAMMA_CRC_ERR       ((CM0_STATUS)(0x9C95 | CM0_FAIL))
#define CM0_CRYPTED_HASH_ERR    ((CM0_STATUS)(0xACCF | CM0_FAIL)) // Name changed 15.01.19. Initial CM0_CRYPTED_CRC_ERR
#define CM0_OPENED_CRC_ERR      ((CM0_STATUS)(0x5F0A | CM0_FAIL))
#define CM0_ROM_CRC_ERR         ((CM0_STATUS)(0x5CAF | CM0_FAIL)) // Value changed 15.01.19. Initial 0x5C0F == CM0_PREINIT_READY
#define CM0_HEALTH_TEST_ERR     ((CM0_STATUS)(0x0CA0 | CM0_FAIL))
#define CM0_RNG_ERR             ((CM0_STATUS)(0x59AC | CM0_FAIL))
#define CM0_RNG_TEST_ERR        ((CM0_STATUS)(0xCAC5 | CM0_FAIL))
#define CM0_NOISE_ERR           ((CM0_STATUS)(0x95CF | CM0_FAIL))
#define CM0_NOISE_TEST_ERR      ((CM0_STATUS)(0xF5C9 | CM0_FAIL))
#define CM0_OTP_R_SN_ERR        ((CM0_STATUS)(0x9635 | CM0_FAIL))
#define CM0_OTP_R_STATUS_ERR    ((CM0_STATUS)(0x6A5F | CM0_FAIL))
#define CM0_OTP_R_CRC_ERR       ((CM0_STATUS)(0x6A59 | CM0_FAIL))
#define CM0_OTP_R_GAMMA_ERR     ((CM0_STATUS)(0x3A9C | CM0_FAIL))
#define CM0_OTP_W_SN_ERR        ((CM0_STATUS)(0xA06A | CM0_FAIL))
#define CM0_OTP_W_STATUS_ERR    ((CM0_STATUS)(0x305A | CM0_FAIL))
#define CM0_OTP_W_CRC_ERR       ((CM0_STATUS)(0x6953 | CM0_FAIL))
#define CM0_OTP_W_GAMMA_ERR     ((CM0_STATUS)(0xA3C9 | CM0_FAIL))

#define CM0_CHK_ERR(status)     (!(status & CM0_OK))

#endif
